-- Tempus - Visum DRIEA SQL import Wrapper

-- ----------------------
-- TABLE road_node
-- ----------------------

INSERT INTO tempus.road_node
SELECT DISTINCT
	n.%(node_NO) AS id,
	false AS bifurcation, -- number of incident edges (to be set below)
	ST_Force3DZ(ST_Transform(ST_SetSrid(geom, 2154), 4326)) AS geom
FROM _tempus_import.node AS n;

-- ----------------------
-- TABLE road_section
-- TABLE road_daily_profile
-- TABLE road_section_speed
-- ----------------------

-- create index to speed up next query
ANALYSE _tempus_import.node;
ANALYSE _tempus_import.link;
CREATE INDEX idx_tempus_import_node_id ON _tempus_import.node(%(node_NO));
CREATE INDEX idx_tempus_import_link_id ON _tempus_import.link(%(link_NO));

-- new validity period for free-flow and loaded regimes
INSERT INTO tempus.road_validity_period (id, name) VALUES (1, 'freeflow');
INSERT INTO tempus.road_validity_period (id, name) VALUES (2, 'loaded');


-- create sequences for auto-incremented ids
DROP SEQUENCE IF EXISTS nb_sections_fflow;
CREATE SEQUENCE nb_sections_fflow MINVALUE 0 START WITH 1;
DROP SEQUENCE IF EXISTS nb_sections_loaded;
CREATE SEQUENCE nb_sections_loaded MINVALUE 0 START WITH 1;
DROP SEQUENCE IF EXISTS nb_profiles;
CREATE SEQUENCE nb_profiles MINVALUE 0 START WITH 11;

-- Modify nb_sections value if more edges have to be added to the network
SELECT setval('nb_sections_fflow', (SELECT GREATEST(MAX(id)+1, nextval('nb_sections_fflow'))-1 FROM tempus.road_section));
SELECT setval('nb_sections_loaded', (SELECT GREATEST(MAX(id)+1, nextval('nb_sections_loaded'))-1 FROM tempus.road_section));
SELECT setval('nb_profiles',(SELECT GREATEST(MAX(profile_id)+1, nextval('nb_profiles'))-1 FROM tempus.road_daily_profile));

-- Create temporary table for database populating (tables road_section, road_daily_profile, and road_section_speed)
CREATE TEMPORARY TABLE full_info
(
    profile_id integer NOT NULL,
    section_id integer NOT NULL,
    period_id integer NOT NULL,
    node_from bigint,
    node_to bigint,
    traffic_rules_ft smallint NOT NULL,
    traffic_rules_tf smallint NOT NULL,
    length double precision NOT NULL,
    road_name varchar(256),
    num_lanes integer,
    car_speed_limit double precision,
    tollway boolean,
    average_speed double precision
);
SELECT AddGeometryColumn('', 'full_info', 'geom', 4326, 'LINESTRING', 3);

-- Insert records into temporary table (from->to direction, free-flow regime)
INSERT INTO
       full_info
SELECT
	nextval('nb_profiles') AS profile_id,
	nextval('nb_sections_fflow') AS section_id,
        1 as period_id,
	l.%(link_FROMNODENO) AS node_from,
	l.%(link_TONODENO) AS node_to,
	CASE WHEN '%(pedestrian)' = ANY(STRING_TO_ARRAY(l.%(link_TSYSSET), ',')) THEN 1 ELSE 0 END
	+ CASE WHEN '%(bike)' = ANY(STRING_TO_ARRAY(l.%(link_TSYSSET), ',')) THEN 2 ELSE 0 END
	+ CASE WHEN '%(car)' = ANY(STRING_TO_ARRAY(l.%(link_TSYSSET), ',')) THEN 4 ELSE 0 END
	+ CASE WHEN '%(taxi)' = ANY(STRING_TO_ARRAY(l.%(link_TSYSSET), ',')) THEN 8 ELSE 0 END
	AS traffic_rules_ft,
	0::int AS traffic_rules_tf,
	CAST(substring(l.%(link_LENGTH), '[0-9]*.[0-9]*') AS double precision)*1000 AS length,
	l.%(link_NOMROUTE) AS road_name,
	l.%(link_NUMLANES) AS num_lanes,
	CAST(substring(l.%(link_V0PRT), '[0-9]*') AS double precision) AS car_speed_limit,
	CAST(l."%(link_TOLL_PRTSYS(V))" AS double precision) > 0 AS tollway,
        CAST(substring(l.%(link_V0PRT), '[0-9]*') AS double precision) AS average_speed,
	ST_Force3DZ(ST_LineMerge(ST_Transform(ST_SetSrid(geom, 2154), 4326))) AS geom
FROM
	_tempus_import.link AS l;

-- Insert records into temporary table (from->to direction, loaded regime)
INSERT INTO
       full_info
SELECT
	nextval('nb_profiles') AS profile_id,
	nextval('nb_sections_loaded') AS section_id,
        2 as period_id,
	l.%(link_FROMNODENO) AS node_from,
	l.%(link_TONODENO) AS node_to,
	CASE WHEN '%(pedestrian)' = ANY(STRING_TO_ARRAY(l.%(link_TSYSSET), ',')) THEN 1 ELSE 0 END
	+ CASE WHEN '%(bike)' = ANY(STRING_TO_ARRAY(l.%(link_TSYSSET), ',')) THEN 2 ELSE 0 END
	+ CASE WHEN '%(car)' = ANY(STRING_TO_ARRAY(l.%(link_TSYSSET), ',')) THEN 4 ELSE 0 END
	+ CASE WHEN '%(taxi)' = ANY(STRING_TO_ARRAY(l.%(link_TSYSSET), ',')) THEN 8 ELSE 0 END
	AS traffic_rules_ft,
	0::int AS traffic_rules_tf,
	CAST(substring(l.%(link_LENGTH), '[0-9]*.[0-9]*') AS double precision)*1000 AS length,
	l.%(link_NOMROUTE) AS road_name,
	l.%(link_NUMLANES) AS num_lanes,
	CAST(substring(l.%(link_V0PRT), '[0-9]*') AS double precision) AS car_speed_limit,
	CAST(l."%(link_TOLL_PRTSYS(V))" AS double precision) > 0 AS tollway,
	CAST(substring(l."%(link_VCUR_PRTSYS(V))", '[0-9]*') AS double precision) AS average_speed,
	ST_Force3DZ(ST_LineMerge(ST_Transform(ST_SetSrid(geom, 2154), 4326))) AS geom
FROM
	_tempus_import.link AS l;

-- Insert records into temporary table (to->from direction, free-flow regime)
INSERT INTO
       full_info
SELECT
	nextval('nb_profiles') AS profile_id,
	nextval('nb_sections_fflow') AS section_id,
        1 as period_id,
	l."%(link_R_FROMNODENO)" AS node_from,
	l.%(link_R_TONODENO) AS node_to,
	CASE WHEN '%(pedestrian)' = ANY(STRING_TO_ARRAY(l.%(link_R_TSYSSET), ',')) THEN 1 ELSE 0 END
	+ CASE WHEN '%(bike)' = ANY(STRING_TO_ARRAY(l.%(link_R_TSYSSET), ',')) THEN 2 ELSE 0 END
	+ CASE WHEN '%(car)' = ANY(STRING_TO_ARRAY(l.%(link_R_TSYSSET), ',')) THEN 4 ELSE 0 END
	+ CASE WHEN '%(taxi)' = ANY(STRING_TO_ARRAY(l.%(link_R_TSYSSET), ',')) THEN 8 ELSE 0 END
	AS traffic_rules_ft,
	0::int AS traffic_rules_tf,
	CAST(substring(l.%(link_R_LENGTH), '[0-9]*.[0-9]*') AS double precision)*1000 AS length,
	l.%(link_R_NOMROUTE) AS road_name,
	l.%(link_R_NUMLANES) AS num_lanes,
	CAST(substring(l.%(link_R_V0PRT), '[0-9]*') AS double precision) AS car_speed_limit,
	CAST(l."%(link_R_TOLL_PRTSYS(V))" AS double precision) > 0 AS tollway,
        CAST(substring(l.%(link_R_V0PRT), '[0-9]*') AS double precision) as average_speed,
	ST_Force3DZ(ST_LineMerge(ST_Transform(ST_SetSrid(geom, 2154), 4326))) AS geom
FROM
	_tempus_import.link AS l;

-- Insert records into temporary table (to->from direction, loaded regime)
INSERT INTO
       full_info
SELECT
	nextval('nb_profiles') AS profile_id,
	nextval('nb_sections_loaded') AS section_id,
        2 as period_id,
	l."%(link_R_FROMNODENO)" AS node_from,
	l.%(link_R_TONODENO) AS node_to,
	CASE WHEN '%(pedestrian)' = ANY(STRING_TO_ARRAY(l.%(link_R_TSYSSET), ',')) THEN 1 ELSE 0 END
	+ CASE WHEN '%(bike)' = ANY(STRING_TO_ARRAY(l.%(link_R_TSYSSET), ',')) THEN 2 ELSE 0 END
	+ CASE WHEN '%(car)' = ANY(STRING_TO_ARRAY(l.%(link_R_TSYSSET), ',')) THEN 4 ELSE 0 END
	+ CASE WHEN '%(taxi)' = ANY(STRING_TO_ARRAY(l.%(link_R_TSYSSET), ',')) THEN 8 ELSE 0 END
	AS traffic_rules_ft,
	0::int AS traffic_rules_tf,
	CAST(substring(l.%(link_R_LENGTH), '[0-9]*.[0-9]*') AS double precision)*1000 AS length,
	l.%(link_R_NOMROUTE) AS road_name,
	l.%(link_R_NUMLANES) AS num_lanes,
	CAST(substring(l.%(link_R_V0PRT), '[0-9]*') AS double precision) AS car_speed_limit,
	CAST(l."%(link_R_TOLL_PRTSYS(V))" AS double precision) > 0 AS tollway,
	CAST(substring(l."%(link_R_VCUR_PRTSYS(V))", '[0-9]*') AS double precision) AS average_speed,
	ST_Force3DZ(ST_LineMerge(ST_Transform(ST_SetSrid(geom, 2154), 4326))) AS geom
FROM
	_tempus_import.link AS l;

-- insert the records into tempus tables and drop temporary table
INSERT INTO tempus.road_section(id, node_from, node_to, traffic_rules_ft, traffic_rules_tf, length, car_speed_limit, road_name, lane, roundabout, tollway, geom)
SELECT DISTINCT
	section_id AS id,
	node_from AS node_from,
	node_to AS node_to,
	traffic_rules_ft AS traffic_rules_ft,
	traffic_rules_tf AS traffic_rules_tf,
	length AS length,
	car_speed_limit AS car_speed_limit,
	road_name AS road_name,
	num_lanes AS lane,
	false AS roundabout,
	tollway AS tollway,
	geom AS geom
FROM full_info
WHERE
	traffic_rules_ft > 0;

INSERT INTO tempus.road_daily_profile (profile_id, begin_time, end_time, speed_rule, average_speed)
SELECT
  profile_id,
  0 as begin_time,
  1440 as end_time,
  5 as speed_rule, -- car
  average_speed
FROM full_info
WHERE traffic_rules_ft >= 4;

INSERT INTO tempus.road_section_speed
SELECT section_id, period_id, profile_id
FROM full_info
WHERE traffic_rules_ft >= 4;

DROP TABLE full_info;

-- Set road_node.bifurcation flag
UPDATE tempus.road_node
SET
        bifurcation = true
WHERE id IN
(
        SELECT
                rn.id AS id
        FROM
                tempus.road_node AS rn,
                tempus.road_section AS rs
        WHERE
                rs.node_from = rn.id
        OR
                rs.node_to = rn.id
        GROUP BY
	        rn.id
        HAVING count(*) > 2
);

-- Set road_section.roundabout flag
UPDATE tempus.road_section
SET
        roundabout = true
WHERE id IN
(
        SELECT
                rs.id AS id
        FROM
                _tempus_import.node AS n,
                tempus.road_section AS rs
        WHERE
                rs.node_from = n.%(node_NO)
	AND
                CAST(n."%(node_CONTROLTYPE)" AS double precision) = 5
);
